/*
   Standard C include file for SIR I/O and projection transformation routines
   with double precision interface

   DGL 25 Mar 2017
   updated by DGL 22 Feb. 2018 + added lowlevel routine prototypes

(c) copyright 2017, 2018 by BYU MERS
*/

#ifndef DSIR_H
#define DSIR_H

#ifndef SIR3_H
#include "sir3.h" /* make sure basis sir routines are defined */
#endif

/* double precision SIR routine function prototypes */
void dpixtolatlon(double x, double y, double *alon, double *alat, 
		 int iopt, float xdeg, float ydeg,
		 float ascale, float bscale, float a0, float b0);
/* given pixel coordinates x,y and projection parameters iopt..b0
   computes the corresponding lon,lat */

void dlatlon2pix(double alon, double alat, double *x, double *y, 
		 int iopt, float xdeg, float ydeg,
		 float ascale, float bscale, float a0, float b0);
/* given a lon and lat and projection parameters iopt..b0
   computes the corresponding x and y values (which may be outside
   of conventional SIR image */

void df2ipix(double x, double y, int *ix, int *iy, int nsx, int nsy);
/* given pixel coordinates x,y as floats and the image size
   compute the corresponding  integer pixel coordinates.  Note that
   pixel coordinates are (1 <= x <= nsx) (1 <= y <= nsy) the fractional
   part of the float is truncated off, thus, (1,1) corresponds to the
   lower-left corner of the lower-left pixel.  A returned
   ix=0 or iy=0 indicates that pixel is outside of image (out of
   range).  If ix and iy are in range (non-zero) lexicographic index
   of pixel (x,y) is index = (y-1)*nsx + x where 1 <= index <= nsx*nsy */

/* ez interface routines */

void dsir_pix2latlon(double x, double y, double *alon, double *alat, sir_head *head);

int dsir_latlon2pix(double alon, double alat, double *x, double *y, sir_head *head);

int dsir_pix(double x, double y, int *ix, int *iy, sir_head *head);

/* low level routines from dsir_geom 
   These are not normally called by the user */

int dintfix(float r);
int ddceil(double r);

void dilambert1(double *lat, double *lon, double x, double y, 
	       float orglat, float orglon, int iopt);
void dipolster(double *alon, double *alat, double x, double y, 
	      float xlam, float slat);
void dieasegrid(int iopt, double *alon, double *alat, 
	       double thelon, double thelat, float ascale);
void diease2grid(int, double*, double*, double, double, float, float);

void dlambert1(double lat, double lon, double *x, double *y, 
	       float orglat, float orglon, int iopt);
void dpolster(double alon, double alat, double *x1, double *y1, float xlam, float slat);
void deasegrid(int iopt, double alon, double alat, 
	       double *thelon, double *thelat, float ascale);
void dease2grid(int iopt, double alon, double alat, 
	       double *thelon, double *thelat, float ascale, float bscale);

double darctand(double y, double x);
void dease2_map_info(int iopt, int isc, int ind, 
		    double *map_equatorial_radius_m, double *map_eccentricity, 
		    double *e2, double *map_reference_latitude, 
		    double *map_reference_longitude, 
		    double *map_second_reference_latitude,double * sin_phi1, 
		    double *cos_phi1, double *kz,
		    double *map_scale, int *bcols, int *brows, 
		    double *r0, double *s0, double *epsilon);
double deaseconv_normalize_degrees(double b);
void dease2sf(int iopt, float ascale, float bscale, double *fmap_scale, 
	     int *bcols, int *brows, double *fr0, double *fs0);

#endif
