function [fmap_scale,bcols,brows,fr0,fs0]=ease2sf(iopt,ascale,bscale)
%
% function [fmap_scale,bcols,brows,fr0,fs0]=ease2sf(iopt,ascale,bscale)
%
% returns selected EASE2 grid information
%
%  inputs (integer values)
%    iopt: projection type 8=EASE2 N, 9-EASE2 S, 10=EASE2 T/M
%    ascale:  scale factor 0..5 grid size is (basesize(ind))/2^isc
%    bscale:  base grid size index   (map units per cell in m)
%    for definitions of ind and isc see EASE2_MAP_INFO
%
%  outputs (real)
%    fmap_scale               EASE2 map projection pixel size (km)
%    bcols, brows,            EASE2 grid size in pixels
%    fr0, fs0                 EASE2 base projection size in pixels
%

% written by D. Long  22 Aug 2019

isc=round(ascale);
ind=round(bscale);

% get base EASE2 map projection parameters
[map_equatorial_radius_m, map_eccentricity, ...
  e2, map_reference_latitude, map_reference_longitude, ...
  map_second_reference_latitude, sin_phi1, cos_phi1, kz, ...
  map_scale, cols, rows, r0, s0, epsilon] = ease2_map_info(iopt, isc, ind);

fmap_scale=map_scale;
bcols=cols;
brows=row;
fr0=r0;
fs0=s0;
